import Ember from 'ember';

const {
  get,
  set,
  computed,
  Mixin,
  isEmpty,
  Logger: { warn }
} = Em

export default Mixin.create({
  /**
   * @private
   *
   * @returns {Object} Returns the index of the focused object and some
   * information around it
   */

  _findIndices (options, focused) {
    let indexOfFocus = options.indexOf(focused)

    let lastIsFocused    = indexOfFocus === (get(options, 'length') - 1)
    let firstIsFocused   = indexOfFocus === 0
    let nothingIsFocused = isEmpty(focused)

    return {
      indexOfFocus,
      firstIsFocused,
      lastIsFocused,
      nothingIsFocused
    }
  },

  /**
   * @private
   *
   * @returns {Object} The object to focus on
   */

  _findNextFocus (direction, options, focused) {
    let nextFocus

    let {
      indexOfFocus,
      firstIsFocused,
      lastIsFocused,
      nothingIsFocused
    } = this._findIndices(options, focused)

    let up   = direction === 'up'
    let down = direction === 'down'

    /*
     * In case the first element is focused, and the direction is
     * up, we move to the last element.
    */

    if ((firstIsFocused || nothingIsFocused) && up) {
      nextFocus = options[options.length - 1]

    } else if ((lastIsFocused || nothingIsFocused) && down) {
      nextFocus = options[0]

    } else {
      let delta

      if (direction === 'up') {
        delta = -1
      } else if (direction === 'down') {
        delta = 1
      }

      nextFocus = options[indexOfFocus + delta]
    }

    return nextFocus
  },

  _move (direction, e) {
    let selectable = get(this, 'selectable')
    let focused    = get(this, 'focused')

    let nextFocus = this._findNextFocus(direction, selectable, focused)

    if (isEmpty(nextFocus)) {
      warn('[en-select] Could not find the next option to focus on')
      return
    }

    this.send('focus', nextFocus, e)
  }
});
